#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "types.h"
#include "proto.h"
#include "select.h"
#include "studentized.h"
#include "print.h"
#include "mean-groups.h"

/* local prototypes */
static int cmp_reals(const void * m1, const void * m2);
static void sort_means(Means list[], matrix_dim howmany);
static matrix_dim previous_group(const chose_t * average, matrix_dim start);

/*                                                                         *\
   -----------------------------------------------------------------------
			       Sorting Means
   -----------------------------------------------------------------------
\*                                                                         */

int cmp_reals(const void * m1, const void * m2) 
{
   if (((Means *)m1)->mean == ((Means *)m2)->mean) { 
      return(0); 
   } else {
      return (((Means *)m1)->mean > ((Means *)m2)->mean) ?(-1) :(+1);
   }
} /* cmp_reals() */


void sort_means(Means      list[],
                matrix_dim howmany) 
{
   qsort(list, (size_t)howmany, sizeof(Means), cmp_reals);
} /* sort_means() */
                 

/*                                                                         *\
   -----------------------------------------------------------------------
                        Computation of Mean Groups
   -----------------------------------------------------------------------
\*                                                                         */

real HSD(const bool       signifdiff,
         const matrix_dim dfPARTI,   
         const matrix_dim dfQUERY,   
         const real       msWITHIN,
         const double     alpha_value,
         const bool       showall)
{
   real Q;  /* Q is the Tukey's honestly significant difference */

   if (signifdiff) {
      if (showall) {
         printf("At least one pair of the participant means differ.");
         NewLine(1, stdout);
         printf("min. signif. diff. = "                             \
                "Q[%lu,(%lu-1)(%lu-1);%.2f] * sqrt(MSW/#queries)",  \
                dfPARTI + 1, dfPARTI + 1, dfQUERY + 1, alpha_value);
         NewLine(1, stdout);
      }
      Q = (real)student(alpha_value, (dfPARTI * dfQUERY), (dfPARTI + 1));
      if (showall) {
         printf("                   ="                                 \
                POS_REAL_SPEC" * sqrt("POS_REAL_SPEC"/"MAT_DIM_SPEC")",\
                Q, msWITHIN, (dfQUERY + 1));
         NewLine(1, stdout);
      }
      Q *= sqrt( msWITHIN / (real)(dfQUERY + 1) );
      if (showall) {
         printf("                   ="POS_REAL_SPEC, Q);
         NewLine(2, stdout);
      }
   } else {
      Q = -2.0;
      if (showall) {
         fputs("No significant difference detected between " \
               "participant means.", stdout);
         NewLine(2, stdout);
      } 
   }
   return(Q);
} /* HSD() */


bool make_means(      Means      average[],
                const string     name[],
                const real       mean[],
                const matrix_dim howmany)
{
   register matrix_dim i;

   for (i=0; i < howmany; i++) {
      average[i].mean = mean[i];
      if (!s_init(&(average[i].groups), name[i], howmany, 0)) {
         register matrix_dim j;

         for (j=0; j < i; j++) {
            s_destroy(&(average[j].groups));
	 }
         return(FALSE);
      } /* for j */
   } /* for i */
   return(TRUE);
} /* make_means() */


void mean_groups(const real       cutoff,  /* < 0 to place all in one group */
                       Means      average[],
                       matrix_dim howmany)
{
 /*                                                                          *
  * If the smallest mean in group x and the largest mean in group x - 1      *
  * differ by less than the LSD, then group x should be part of group x - 1. *
  * The largest mean in group x is mean x.  We can collapse groups x and     *
  * x - 1 by removing group x, because of the order in which groups are      *
  * formed (the g and h loops below).                                        *
  *                                                                          */

   const    char       proc[] = "mean_groups";
   extern   char       errorstring[];
   register matrix_dim g; 
   register matrix_dim h;
            chose_t    true_groups;
   
   for (g=0; g < howmany; g++) {
      for (h=0; h < howmany; h++) {
         if (!s_append(NULL, &average[g].groups, FALSE)) {
            sprintf(errorstring,                                     \
                    "cannot make mean groups -- "                    \
                    "error from s_append(NULL, average["MAT_DIM_SPEC \
                    "].groups, FALSE", g);
            swerr(proc, errorstring, 0);
            return;
	 }
      }
   }
   sort_means(average, howmany);
   if (cutoff < 0.0) { /* all in one group */
      for (g=0; g < howmany; g++) {
         s_assign(&(average[g].groups), 0, TRUE); 
      }
   } else { 
      bool    diff;

      if (! s_init(&true_groups, "real mean groups", howmany, 0)) {
         swerr(proc, "Not enough memory to make groups", 0);
         return; 
      }
      /* assign to groups before collapsing groups */
      for (g=0; g < howmany; g++) {
         s_append(NULL, &true_groups, (0 == g));
         diff = FALSE; /* no difference detected yet */
         for (h=g+1; !diff && (h < howmany); h++) {
            if ((average[g].mean - average[h].mean) <= cutoff) {
               s_assign(&(average[g].groups), g, TRUE);  /* g is in group G */
               s_assign(&(average[h].groups), g, TRUE);  /* h is in group G */
	    } else {
               diff = TRUE;
	    }
         } /* for h */
         if (0 != s_count(&average[g].groups)) { 
            bool       should_be_group;
            matrix_dim previous = 0;
            matrix_dim last     = (diff)?(h - 2):(h - 1);

            previous = (0 != g) ?previous_group(&true_groups, g) :0;
            should_be_group = ((0 == g) ||                                \
                  ((average[previous].mean - average[last].mean) > cutoff));
            s_assign(&true_groups, g, should_be_group);
         } else {
            s_assign(&(average[g].groups), g, TRUE);
            s_assign(&true_groups, g, TRUE);
         }
      } /* for g */

      /* now collapse groups */
      /*   for every mean: the mean's group is itself intersected with 
                     the true_groups vector */
      for (g=1; g < howmany; g++) {
         if (FALSE == s_value(&true_groups, g)) {
            for (h=0; h < howmany; h++) {
               s_assign(&average[h].groups, g, FALSE);
            }         
	 }
      }      
   }
} /* mean_groups() */


matrix_dim previous_group(const chose_t *  groups,
                                matrix_dim start)
{
   extern       char errorstring[];
          const char proc[] = "previous_group";
                bool found  = FALSE;

   sprintf(errorstring, "can't find group previous to "MAT_DIM_SPEC, start);
   do {
      found = s_value(groups, start - 1);
      if (! found) {
         start--;
      }
   } while ((! found) && (start > 0));
   if (! found) {
      swerr(proc, errorstring, 1); /* NOTREACHED */
   }
   return(start - 1);
} /* previous_group() */

/*                                                                         *\
   -----------------------------------------------------------------------

   -----------------------------------------------------------------------
\*                                                                         */

