#include "studentized.h"

extern void swerr(const char proc[], const char * err_mess, \
                  const int error_level);
static double qtrng_(double *p, double *v, double *r, long int *ifault);


double student(double        alpha, 
               unsigned long dof,
               unsigned long numsamp)
{
   const char     proc[] = "student";
         double   result;
         double   deg_freedom = (double)dof;
         double   number      = (double)numsamp;
         long int ifault;

   if (alpha < 0.9) {
      alpha = 1.0L - alpha;
   }

   result = qtrng_(&alpha, &deg_freedom, &number, &ifault);

   if (!ifault) {
      return(result);
   } else {
      switch(ifault) {
         case 1: swerr(proc, "error from qtrng(): "
                             "dof < 1 or number of samples < 2", 0);
               break;
         case 2: swerr(proc, "error from qtrng(): "
                             "alpha must be between 0.9 and 0.99", 0);
               break;
         case 9: swerr(proc, "error from qtrng(): "
                             "error passed up from gauinv()", 0);
               break;
        default: swerr(proc, "error from qtrng(): unknown error", 0);
               break;
      } /* switch */
      return (-1.0);
   }
} /* student() */


/*
   --------------------------------------------------------------------------
   * Translations of 
      ALGORITHM AS 190 APPL. STATIST. (1983) VOL.32, No.2
      obtained from http://lib.stat.cmu.edu/apstat/190 
     and
      ALGORITHM AS 111 APPL. STATIST., VOL.26, 118-121, 1977.
      obtained from http://lib.stat.cmu.edu/apstat/111
     and
      Algorithm AS66 Applied Statistics (1973) vol22 no.3
      obtained from http://lib.stat.cmu.edu/apstat/66
     from FORTRAN to C using the AT&T f2c software.

   * According to StatLib qtrng_() is an extract from 
      Algorithm AS190 from Applied Statistics (1983) v.22 no.2 
      incorporating corrections from Applied Statistics (1985) v.34 no.2 
     and 
      ppnd_() is 
      Algorithm AS 111 from  Applied Statistics (1977) v.26, pp. 118-121
     and 
      alnorm_() is
      Algorithm AS66 from Applied Statistics (1973) v.22 no.3 

   * AS190 is really from v.32 no.2 (pp. 204 -- 210) and 
     the corrections are from v.34 no.1 (p.104)

   * I also made a change to the original FORTRAN code of AS190.  A line 
     was too long to be legal so I continued it.  The bug has been reported
     and corrected as of 9 Feb 1995.

   * f2c command-line arguments:
      -C -onetrip -A -c -ext -g -P
     [NB: line numbers were removed later]

   * f2c copyright notice:
      Copyright 1990 - 1995 by AT&T Bell Laboratories and Bellcore.
   
      Permission to use, copy, modify, and distribute this software
      and its documentation for any purpose and without fee is hereby
      granted, provided that the above copyright notice appear in all
      copies and that both that the copyright notice and this
      permission notice and warranty disclaimer appear in supporting
      documentation, and that the names of AT&T Bell Laboratories or
      Bellcore or any of their entities not be used in advertising or
      publicity pertaining to distribution of the software without
      specific, written prior permission.
   
      AT&T and Bellcore disclaim all warranties with regard to this
      software, including all implied warranties of merchantability
      and fitness.  In no event shall AT&T or Bellcore be liable for
      any special, indirect or consequential damages or any damages
      whatsoever resulting from loss of use, data or profits, whether
      in an action of contract, negligence or other tortious action,
      arising out of or in connection with the use or performance of
      this software.
   -------------------------------------------------------------------------- */
/* as190.f -- translated by f2c (version 19950201).
   You must link the resulting object file with the libraries:
	-lf2c -lm   (in that order)
*/

#include "f2c.h"
static double alnorm_(double *x, logical *upper);
static doublereal ppnd_(doublereal *p, integer *ier);
static doublereal prtrng_(doublereal *q, doublereal *v, doublereal *r, \
        integer * ifault);
static doublereal qtrng0_(doublereal *, doublereal *, doublereal *, \
	    integer *);


/* Table of constant values */

static logical c_true = TRUE_;

/*<       double precision function prtrng(q, v, r, ifault) >*/
doublereal prtrng_(doublereal *q, doublereal *v, doublereal *r, integer *
	ifault)
{
    /* Initialized data */

    static doublereal pcutj = 3e-5; /*2e-6;*/
          /* original value of pcutj = 3e-5 */
    static doublereal cv1 = .193064705;
    static doublereal cv2 = .293525326;
    static doublereal cvmax = .39894228;
    static doublereal cv[4] = { .318309886,-.00268132716,.00347222222,
	    .0833333333 };
    static integer jmin = 3;
    static integer jmax = 15; /*20;*/
          /* original value of jmax = 15 */
    static integer kmin = 7;
    static integer kmax = 15; /*20;*/
          /* original value of kmax = 15 */
    static doublereal pcutk = 1e-4; /*1e-6;*/ 
          /* original value of pcutk = 1e-4 */
    static doublereal step = .45; /*.3375; */
          /* original value of step = .45 */
    static doublereal vmax = 120.;
    static doublereal zero = 0.;
    static doublereal fifth = .2;
    static doublereal half = .5;
    static doublereal one = 1.;
    static doublereal two = 2.;

    /* System generated locals */
    integer i__1, i__2, i__3;
    doublereal ret_val, d__1;

    /* Builtin functions */
    double pow_dd(doublereal *, doublereal *);
    double log(doublereal);
    double sqrt(doublereal);
    integer s_rnge(char *, integer, char *, integer);
    double exp(doublereal);

    /* Local variables */
    static doublereal gmid;
    static integer jump;
    static doublereal c, g, h;
    static integer j, k;
    static doublereal x, gstep, r1, w0, v2, gk, hj;
    static integer jj;
    static doublereal pj, pk, qw[30/*240*/], pz, vw[30/*240*/];
          /* original dimension of qw and vw were both 30 */
    extern doublereal alnorm_(doublereal *, logical *);
    static doublereal pk1, pk2, ehj;

/*<       implicit double precision (a-h, o-z) >*/

/*        Algorithm AS 190  Appl. Statist. (1983) Vol.32, No. 2 */
/*        Incorporates corrections from Appl. Statist. (1985) Vol.34 (1) 
*/

/*        Evaluates the probability from 0 to q for a studentized */
/*        range having v degrees of freedom and r samples. */

/*        Uses subroutine ALNORM = algorithm AS66. */

/*        Arrays vw and qw store transient values used in the */
/*        quadrature summation.  Node spacing is controlled by */
/*        step.  pcutj and pcutk control truncation. */
/*        Minimum and maximum number of steps are controlled by */
/*        jmin, jmax, kmin and kmax.  Accuracy can be increased */
/*        by use of a finer grid - Increase sizes of arrays vw */
/*        and qw, and jmin, jmax, kmin, kmax and 1/step proportionally. */

/*<    >*/
/*<       double precision g, gmid, r1, c, h, v2, gstep, pk1, pk2, gk, pk >*/
/*<       double precision w0, pz, x, hj, ehj, pj >*/
/*<    >*/
/*<       data jmin, jmax, kmin, kmax/3, 15, 7, 15/ >*/

/*        Check initial values */
/*<       prtrng = zero >*/
    ret_val = zero;
/*<       ifault = 0 >*/
    *ifault = 0;
/*<       if (v .lt. one .or. r .lt. two) ifault = 1 >*/
    if (*v < one || *r < two) {
	*ifault = 1;
    }
/*<       if (q .le. zero .or. ifault .eq. 1) goto 99 >*/
    if (*q <= zero || *ifault == 1) {
	goto L99;
    }

/*        Computing constants, local midpoint, adjusting steps. */

/*<       g = step * r ** (-fifth) >*/
    d__1 = -fifth;
    g = step * pow_dd(r, &d__1);
/*<       gmid = half * log(r) >*/
    gmid = half * log(*r);
/*<       r1 = r - one >*/
    r1 = *r - one;
/*<       c = log(r * g * cvmax) >*/
    c = log(*r * g * cvmax);
/*<       if(v.gt.vmax) goto 20 >*/
    if (*v > vmax) {
	goto L20;
    }
/*<       h = step * v ** (-half) >*/
    d__1 = -half;
    h = step * pow_dd(v, &d__1);
/*<       v2 = v * half >*/
    v2 = *v * half;
/*<       if (v .eq. one) c = cv1 >*/
    if (*v == one) {
	c = cv1;
    }
/*<       if (v .eq. two) c = cv2 >*/
    if (*v == two) {
	c = cv2;
    }
/*<    >*/
    if (! (*v == one || *v == two)) {
	c = sqrt(v2) * cv[0] / (one + ((cv[1] / v2 + cv[2]) / v2 + cv[3]) / 
		v2);
    }
/*<       c = log(c * r * g * h) >*/
    c = log(c * *r * g * h);

/*        Computing integral */
/*        Given a row k, the procedure starts at the midpoint and works */
/*        outward (index j) in calculating the probability at nodes */
/*        symmetric about the midpoint.  The rows (index k) are also */
/*        processed outwards symmetrically about the midpoint.  The */
/*        centre row is unpaired. */

/*<    20 gstep = g >*/
L20:
    gstep = g;
/*<       qw(1) = -one >*/
    qw[0] = -one;
/*<       qw(jmax + 1) = -one >*/
    qw[(i__1 = jmax) < 30 && 0 <= i__1 ? i__1 : s_rnge("qw", i__1, "prtrng_", 
	    65L)] = -one;
/*<       pk1 = one >*/
    pk1 = one;
/*<       pk2 = one >*/
    pk2 = one;
/*<       do 28 k = 1, kmax >*/
    i__1 = kmax;
    for (k = 1; k <= i__1 || k == 1; ++k) {
/*<         gstep = gstep - g >*/
	gstep -= g;
/*<    21   gstep = -gstep >*/
L21:
	gstep = -gstep;
/*<         gk = gmid + gstep >*/
	gk = gmid + gstep;
/*<         pk = zero >*/
	pk = zero;
/*<         if (pk2 .le. pcutk .and. k .gt. kmin) goto 26 >*/
	if (pk2 <= pcutk && k > kmin) {
	    goto L26;
	}
/*<         w0 = c - gk * gk * half >*/
	w0 = c - gk * gk * half;
/*<         pz = alnorm(gk, .true.) >*/
	pz = alnorm_(&gk, &c_true);
/*<         x = alnorm(gk - q, .true.) - pz >*/
	d__1 = gk - *q;
	x = alnorm_(&d__1, &c_true) - pz;
/*<         if (x .gt. zero) pk = exp(w0 + r1 * log(x)) >*/
	if (x > zero) {
	    pk = exp(w0 + r1 * log(x));
	}
/*<         if (v .gt. vmax) goto 26 >*/
	if (*v > vmax) {
	    goto L26;
	}
/*<         jump = -jmax >*/
	jump = -jmax;
/*<    22   jump = jump + jmax >*/
L22:
	jump += jmax;
/*<         do 24 j = 1, jmax >*/
	i__2 = jmax;
	for (j = 1; j <= i__2 || j == 1; ++j) {
/*<           jj = j + jump >*/
	    jj = j + jump;
/*<           if (qw(jj) .gt. zero) goto 23 >*/
	    if (qw[(i__3 = jj - 1) < 30 && 0 <= i__3 ? i__3 : s_rnge("qw", 
		    i__3, "prtrng_", 85L)] > zero) {

		goto L23;
	    }
/*<           hj = h * j >*/
	    hj = h * j;
/*<           if (j .lt. jmax) qw(jj + 1) = -one >*/
	    if (j < jmax) {
		qw[(i__3 = jj) < 30 && 0 <= i__3 ? i__3 : s_rnge("qw", i__3, 
			"prtrng_", 87L)] = -one;
	    }
/*<           ehj = exp(hj) >*/
	    ehj = exp(hj);
/*<           qw(jj) = q * ehj >*/
	    qw[(i__3 = jj - 1) < 30 && 0 <= i__3 ? i__3 : s_rnge("qw", i__3, 
		    "prtrng_", 89L)] = *q * ehj;
/*<           vw(jj) = v * (hj + half - ehj * ehj * half) >*/
	    vw[(i__3 = jj - 1) < 30 && 0 <= i__3 ? i__3 : s_rnge("vw", i__3, 
		    "prtrng_", 90L)] = *v * (hj + half - ehj * ehj * half);
/*<    23     pj = zero >*/
L23:
	    pj = zero;
/*<           x = alnorm(gk - qw(jj), .true.) - pz >*/
	    d__1 = gk - qw[(i__3 = jj - 1) < 30 && 0 <= i__3 ? i__3 : s_rnge(
		    "qw", i__3, "prtrng_", 93L)];
	    x = alnorm_(&d__1, &c_true) - pz;
/*<           if (x .gt. zero) pj = exp(w0 + vw(jj) + r1 * log(x)) >*/
	    if (x > zero) {
		pj = exp(w0 + vw[(i__3 = jj - 1) < 30 && 0 <= i__3 ? i__3 : 
			s_rnge("vw", i__3, "prtrng_", 94L)] + r1 * log(x));
	    }
/*<           pk = pk + pj >*/
	    pk += pj;
/*<           if (pj .gt. pcutj) goto 24 >*/
	    if (pj > pcutj) {
		goto L24;
	    }
/*<           if (jj .gt. jmin .or. k .gt. kmin) goto 25 >*/
	    if (jj > jmin || k > kmin) {
		goto L25;
	    }
/*<    24   continue >*/
L24:
	    ;
	}
/*<    25   h = -h >*/
L25:
	h = -h;
/*<         if (h .lt. zero) goto 22 >*/
	if (h < zero) {
	    goto L22;
	}
L26:
	ret_val += pk;
/*<    >*/
	if (k > kmin && pk <= pcutk && pk1 <= pcutk) {
	    goto L99;
	}
/*<         pk2 = pk1 >*/
	pk2 = pk1;
/*<         pk1 = pk >*/
	pk1 = pk;
/*<         if (gstep .gt. zero) goto 21 >*/
	if (gstep > zero) {
	    goto L21;
	}
/*<    28 continue >*/
/* L28: */
    }
/*<    99 return >*/
L99:
    return ret_val;
/*<       end >*/
} /* prtrng_ */





/*<       double precision function qtrng(p, v, r, ifault) >*/
doublereal qtrng_(doublereal *p, doublereal *v, doublereal *r, integer *
	ifault)
{
    /* Initialized data */

    static integer jmax = 32;        /* original value of jmax = 8 */
    static doublereal pcut = .00001; /* original value of pcut = .001 */
    static doublereal p75 = .75;
    static doublereal p80 = .8;
    static doublereal p90 = .9;
    static doublereal p99 = .99;
    static doublereal p995 = .995;
    static doublereal p175 = 1.75;
    static doublereal one = 1.;
    static doublereal two = 2.;
    static doublereal five = 5.;
    static doublereal eps = 1e-4;

    /* System generated locals */
    integer i__1;
    doublereal ret_val, d__1;

    /* Local variables */
    static doublereal d;
    static integer j;
    static doublereal e1, e2, p1, q1, q2, p2;
    extern doublereal qtrng0_(doublereal *, doublereal *, doublereal *, 
	    integer *);
    static integer nfault;
    extern doublereal prtrng_(doublereal *, doublereal *, doublereal *, 
	    integer *);

/*<       implicit double precision (a-h, o-z) >*/

/*        Algorithm AS 190.1  Appl. Statist. (1983) Vol.32, No. 2 */

/*        Approximates the quantile p for a studentized range */
/*        distribution having v degrees of freedom and r samples */
/*        for probability p, p.ge.0.90 .and. p.le.0.99. */

/*        Uses functions  alnorm, ppnd, prtrng and qtrng0 - */
/*        Algorithms AS 66, AS 111, AS 190 and AS 190.2 */

/*<       double precision p, v, r, pcut, p75, p80, p90, p99, p995 >*/
/*<       double precision p175, one, two, five >*/
/*<       double precision q1, p1, q2, p2, e1, e2 >*/
/*<       double precision eps >*/
/*<    >*/
/*<       data eps/1.0d-4/ >*/

/*        Check input parameters */

/*<       ifault = 0 >*/

    *ifault = 0;
/*<       nfault = 0 >*/

    nfault = 0;
/*<       if (v .lt. one .or. r.lt. two) ifault = 1 >*/

    if (*v < one || *r < two) {

	*ifault = 1;

    }
/*<       if (p .lt. p90 .or. p .gt. p99) ifault = 2 >*/

    if (*p < p90 || *p > p99) {

	*ifault = 2;

    }
/*<       if (ifault .ne. 0) goto 99 >*/

    if (*ifault != 0) {

	goto L99;

    }

/*        Obtain initial values */

/*<       q1 = qtrng0(p, v, r, nfault) >*/

    q1 = qtrng0_(p, v, r, &nfault);
/*<       if (nfault .ne. 0) goto 99 >*/

    if (nfault != 0) {

	goto L99;

    }
/*<       p1 = prtrng(q1, v, r, nfault) >*/

    p1 = prtrng_(&q1, v, r, &nfault);
/*<       if (nfault .ne. 0) goto 99 >*/

    if (nfault != 0) {

	goto L99;

    }
/*<       qtrng = q1 >*/

    ret_val = q1;
/*<       if (abs(p1-p) .lt. pcut) goto 99 >*/

    if ((d__1 = p1 - *p, abs(d__1)) < pcut) {

	goto L99;

    }
/*<       if (p1 .gt. p) p1 = p175 * p - p75 * p1 >*/

    if (p1 > *p) {

	p1 = p175 * *p - p75 * p1;

    }
/*<       if (p1 .lt. p) p2 = p + (p - p1) * (one - p) / (one - p1) * p75 >*/

    if (p1 < *p) {

	p2 = *p + (*p - p1) * (one - *p) / (one - p1) * p75;

    }
/*<       if (p2 .lt. p80) p2 = p80 >*/

    if (p2 < p80) {

	p2 = p80;

    }
/*<       if (p2 .gt. p995) p2 = p995 >*/

    if (p2 > p995) {

	p2 = p995;

    }
/*<       q2 = qtrng0(p2, v, r, nfault) >*/

    q2 = qtrng0_(&p2, v, r, &nfault);
/*<       if (nfault .ne. 0) goto 99 >*/

    if (nfault != 0) {

	goto L99;

    }

/*        Refine approximation */

/*<       do 14 j = 2, jmax >*/

    i__1 = jmax;

    for (j = 2; j <= i__1 || j == 2; ++j) {
/*<         p2 = prtrng(q2, v, r, nfault) >*/

	p2 = prtrng_(&q2, v, r, &nfault);
/*<         if (nfault .ne. 0) goto 99 >*/

	if (nfault != 0) {

	    goto L99;

	}
/*<         e1 = p1 - p >*/

	e1 = p1 - *p;
/*<         e2 = p2 - p >*/

	e2 = p2 - *p;
/*<         qtrng = (q1 + q2) / two >*/

	ret_val = (q1 + q2) / two;
/*<         d = e2 - e1 >*/

	d = e2 - e1;
/*<         if (abs(d) .gt. eps) qtrng = (e2 * q1 - e1 * q2) / d >*/

	if (abs(d) > eps) {

	    ret_val = (e2 * q1 - e1 * q2) / d;

	}
/*<         if(abs(e1) .lt. abs(e2)) goto 12 >*/

	if (abs(e1) < abs(e2)) {

	    goto L12;

	}
/*<         q1 = q2 >*/

	q1 = q2;
/*<         p1 = p2 >*/

	p1 = p2;
/*<    12   if (abs(p1 - p) .lt. pcut * five) goto 99 >*/

L12:

	if ((d__1 = p1 - *p, abs(d__1)) < pcut * five) {

	    goto L99;

	}
/*<         q2 = qtrng >*/

	q2 = ret_val;
/*<    14 continue >*/

/* L14: */

    }

/*<    99 if (nfault .ne. 0) ifault = 9 >*/

L99:

    if (nfault != 0) {

	*ifault = 9;

    }
/*<       return >*/

    return ret_val;
/*<       end >*/
} /* qtrng_ */



/*<       double precision function qtrng0(p, v, r, ifault) >*/
doublereal qtrng0_(doublereal *p, doublereal *v, doublereal *r, integer *
	ifault)
{
    /* Initialized data */

    static doublereal vmax = 120.;
    static doublereal half = .5;
    static doublereal one = 1.;
    static doublereal four = 4.;
    static doublereal c1 = .8843;
    static doublereal c2 = .2368;
    static doublereal c3 = 1.214;
    static doublereal c4 = 1.208;
    static doublereal c5 = 1.4142;

    /* System generated locals */
    doublereal ret_val, d__1;

    /* Builtin functions */
    double log(doublereal);

    /* Local variables */
    extern doublereal ppnd_(doublereal *, integer *);
    static doublereal q, t;

/*<       implicit double precision (a-h, o-z) >*/

/*        Algorithm AS 190.2  Appl. Statist. (1983) Vol.32, No.2 */

/*        Calculates an initial quantile p for a studentized range */
/*        distribution having v degrees of freedom and r samples */
/*        for probability p, p.gt.0.80 .and. p.lt.0.995. */

/*        Uses function ppnd - Algorithm AS 111 */

/*<       double precision p, v, r, q, t, vmax, half, one, four, c1, c2, c3 >*/
/*<       double precision c4, c5 >*/
/*<    >*/

/*<       t=ppnd(half + half * p,ifault) >*/

    d__1 = half + half * *p;

    t = ppnd_(&d__1, ifault);
/*<       if (v .lt. vmax) t = t + (t * t* t + t) / v / four >*/

    if (*v < vmax) {

	t += (t * t * t + t) / *v / four;

    }
/*<       q = c1 - c2 * t >*/

    q = c1 - c2 * t;
/*<       if (v .lt. vmax) q = q - c3 / v + c4 * t / v >*/

    if (*v < vmax) {

	q = q - c3 / *v + c4 * t / *v;

    }
/*<       qtrng0 = t * (q * log(r - one) + c5) >*/

    ret_val = t * (q * log(*r - one) + c5);
/*<       return >*/

    return ret_val;
/*<       end >*/
} /* qtrng0_ */





/*
   --------------------------------------------------------------------------

   * A translation of http://lib.stat.cmu.edu/apstat/111 from FORTRAN to C
   using the AT&T f2c software.

   * According to StatLib this is
     Algorithm AS 111 from  Applied Statistics (1977) v.26, pp. 118-121

   * f2c commandline arguments
     -C -onetrip -A -c -ext -g -P

   --------------------------------------------------------------------------
*/
/* 111.f -- translated by f2c (version 19950201).
   You must link the resulting object file with the libraries:
	-lf2c -lm   (in that order)
*/


/*<         DOUBLE PRECISION FUNCTION PPND(P,IER) >*/
doublereal ppnd_(doublereal *p, integer *ier)
{
    /* Initialized data */

    static doublereal split = .42;
    static doublereal c0 = -2.78718931138;
    static doublereal c1 = -2.29796479134;
    static doublereal c2 = 4.85014127135;
    static doublereal c3 = 2.32121276858;
    static doublereal d1 = 3.54388924762;
    static doublereal d2 = 1.63706781897;
    static doublereal zero = 0.;
    static doublereal one = 1.;
    static doublereal half = .5;
    static doublereal a0 = 2.50662823884;
    static doublereal a1 = -18.61500062529;
    static doublereal a2 = 41.39119773534;
    static doublereal a3 = -25.44106049637;
    static doublereal b1 = -8.4735109309;
    static doublereal b2 = 23.08336743743;
    static doublereal b3 = -21.06224101826;
    static doublereal b4 = 3.13082909833;

    /* System generated locals */
    doublereal ret_val;

    /* Builtin functions */
    double log(doublereal), sqrt(doublereal);

    /* Local variables */
    static doublereal q, r;


/*       ALGORITHM AS 111, APPL.STATIST., VOL.26, 118-121, 1977. */

/*       PRODUCES NORMAL DEVIATE CORRESPONDING TO LOWER TAIL AREA = P. */

/*       See also AS 241 which contains alternative routines accurate to 
*/
/*       about 7 and 16 decimal digits. */

/*<         IMPLICIT DOUBLE PRECISION (A-H,O-Z) >*/
/*<         DATA SPLIT/0.42D0/ >*/
/*<    >*/
/*<         DATA ZERO/0.D0/, ONE/1.D0/, HALF/0.5D0/ >*/

/*<         IER = 0 >*/

    *ier = 0;
/*<         Q = P-HALF >*/

    q = *p - half;
/*<         IF (ABS(Q).GT.SPLIT) GO TO 10 >*/

    if (abs(q) > split) {

	goto L10;

    }

/*       0.08 < P < 0.92 */

/*<         R = Q*Q >*/

    r = q * q;
/*<    >*/

    ret_val = q * (((a3 * r + a2) * r + a1) * r + a0) / ((((b4 * r + b3) * r 
	    + b2) * r + b1) * r + one);
/*<         RETURN >*/

    return ret_val;

/*       P < 0.08 OR P > 0.92, SET R = MIN(P,1-P) */

/*<    10   R = P >*/

L10:

    r = *p;
/*<         IF (Q.GT.ZERO) R = ONE-P >*/

    if (q > zero) {

	r = one - *p;

    }
/*<         IF (R.LE.ZERO) GO TO 20 >*/

    if (r <= zero) {

	goto L20;

    }
/*<         R = SQRT(-LOG(R)) >*/

    r = sqrt(-log(r));
/*<         PPND = (((C3*R + C2)*R + C1)*R + C0)/((D2*R + D1)*R + ONE) >*/

    ret_val = (((c3 * r + c2) * r + c1) * r + c0) / ((d2 * r + d1) * r + one);
/*<         IF (Q.LT.ZERO) PPND = -PPND >*/

    if (q < zero) {

	ret_val = -ret_val;

    }
/*<         RETURN >*/

    return ret_val;
/*<    20   IER = 1 >*/

L20:

    *ier = 1;
/*<         PPND = ZERO >*/

    ret_val = zero;
/*<         RETURN >*/

    return ret_val;
/*<         END >*/
} /* ppnd_ */



/* as66.f -- translated by f2c (version 19950201).
   You must link the resulting object file with the libraries:
	-lf2c -lm   (in that order)
*/


/*This file includes the Applied Statistics algorithm AS 66 for calculating*/
/*the tail area under the normal curve, and two alternative routines which*/
/*give higher accuracy.   The latter have been contributed by Alan Miller of*/
/* CSIRO Division of Mathematics & Statistics, Clayton, Victoria.   Notice */
/* that each function or routine has different call arguments. */


/*<       double precision function alnorm(x,upper) >*/
doublereal alnorm_(doublereal *x, logical *upper)
{
    /* Initialized data */

    static doublereal zero = 0.;
    static doublereal a1 = 5.75885480458;
    static doublereal a2 = 2.62433121679;
    static doublereal a3 = 5.92885724438;
    static doublereal b1 = -29.8213557807;
    static doublereal b2 = 48.6959930692;
    static doublereal c1 = -3.8052e-8;
    static doublereal c2 = 3.98064794e-4;
    static doublereal c3 = -.151679116635;
    static doublereal c4 = 4.8385912808;
    static doublereal c5 = .742380924027;
    static doublereal one = 1.;
    static doublereal c6 = 3.99019417011;
    static doublereal d1 = 1.00000615302;
    static doublereal d2 = 1.98615381364;
    static doublereal d3 = 5.29330324926;
    static doublereal d4 = -15.1508972451;
    static doublereal d5 = 30.789933034;
    static doublereal half = .5;
    static doublereal ltone = 7.;
    static doublereal utzero = 18.66;
    static doublereal con = 1.28;
    static doublereal p = .398942280444;
    static doublereal q = .39990348504;
    static doublereal r = .398942280385;

    /* System generated locals */
    doublereal ret_val;

    /* Builtin functions */
    double exp(doublereal);

    /* Local variables */
    static doublereal y, z;
    static logical up;


/*         Algorithm AS66 Applied Statistics (1973) vol22 no.3 */

/*       Evaluates the tail area of the standardised normal curve */
/*       from x to infinity if upper is .true. or */
/*       from minus infinity to x if upper is .false. */

/*<       double precision zero,one,half >*/
/*<       double precision con,z,y,x >*/
/*<       double precision p,q,r,a1,a2,a3,b1,b2,c1,c2,c3,c4,c5,c6 >*/
/*<       double precision d1,d2,d3,d4,d5 >*/
/*<       logical upper,up >*/
/* *** machine dependent constants */
/*<       double precision ltone,utzero >*/
/*<       data zero/0.0d0/, one/1.0d0/, half/0.5d0/ >*/
/*<       data ltone/7.0d0/,utzero/18.66d0/ >*/
/*<       data con/1.28d0/ >*/
/*<       data p/0.398942280444d0/,q/0.39990348504d0/,r/0.398942280385d0/ >*/
/*<       data a1/5.75885480458d0/,a2/2.62433121679d0/,a3/5.92885724438d0/ >*/
/*<       data b1/-29.8213557807d0/,b2/48.6959930692d0/ >*/
/*<       data c1/-3.8052d-8/,c2/3.98064794d-4/,c3/-0.151679116635d0/ >*/
/*<       data c4/4.8385912808d0/,c5/0.742380924027d0/,c6/3.99019417011d0/ >*/
/*<       data d1/1.00000615302d0/,d2/1.98615381364d0/,d3/5.29330324926d0/ >*/
/*<       data d4/-15.1508972451d0/,d5/30.789933034d0/ >*/

/*<       up=upper >*/

    up = *upper;
/*<       z=x >*/

    z = *x;
/*<       if(z.ge.zero)goto 10 >*/

    if (z >= zero) {

	goto L10;

    }
/*<       up=.not.up >*/

    up = ! up;
/*<       z=-z >*/

    z = -z;
/*<    10 if(z.le.ltone.or.up.and.z.le.utzero)goto 20 >*/

L10:

    if (z <= ltone || up && z <= utzero) {

	goto L20;

    }
/*<       alnorm=zero >*/

    ret_val = zero;
/*<       goto 40 >*/

    goto L40;
/*<    20 y=half*z*z >*/

L20:

    y = half * z * z;
/*<       if(z.gt.con) goto 30 >*/

    if (z > con) {

	goto L30;

    }

/*<       alnorm=half-z*(p-q*y/(y+a1+b1/(y+a2+b2/(y+a3)))) >*/

    ret_val = half - z * (p - q * y / (y + a1 + b1 / (y + a2 + b2 / (y + a3)))
	    );
/*<       goto 40 >*/

    goto L40;
/*<    >*/

L30:

    ret_val = r * exp(-y) / (z + c1 + d1 / (z + c2 + d2 / (z + c3 + d3 / (z + 
	    c4 + d4 / (z + c5 + d5 / (z + c6))))));
/*<    40 if(.not.up)alnorm=one-alnorm >*/

L40:

    if (! up) {

	ret_val = one - ret_val;

    }
/*<       return >*/

    return ret_val;
/*<       end >*/
} /* alnorm_ */

