
/* util.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <limits.h>
#include "types.h"
#include "proto.h"

#define MAX_ERRSTR  (1024 + MAXFNAMELEN)

/* Global Variables */
/* exported globabl variables */
   const char       PROMPT[]    = "Type the number of your selection " \
                                         "and press Return ";

/* module specific globals */
   static int open_files = 0;
#if defined(DEBUG) && defined(SHOW_OPEN_FILES)
   static char open_names[70][1024];
#endif


/*                                                                         *\
   -----------------------------------------------------------------------
		  String (Filename) Manipulation Routines
   -----------------------------------------------------------------------
\*                                                                         */


char chop(string s) {
   long len = strlen(s);
   char t   = s[len];
   
   s[len - 1] = '\0';
   return t;
}


bool ReadString(      char         buffer[],
                const unsigned int length) {

   const char proc[] = "ReadString";

   if (fgets(buffer, (length < INT_MAX)?(int)length:INT_MAX, stdin) != NULL) {
      return(TRUE);
   } else {
      buffer[0] = '\0';
      swerr(proc, "error reading input -- assuming nothing read", 0);
      if (feof(stdin) || ferror(stdin)) {
         return(FALSE);
      } else { /* This should never happen, but ... */
         return(TRUE);
      }
   }
} /* ReadString() */


void GetStringFromUser(      char         instring[],
                       const unsigned int maxlen,
                       const string       prompt) {

   const char proc[] = "GetStringFromUser";

   fputs(prompt, stdout);
   fflush(stdout);
   if ((FALSE == ReadString(instring, maxlen)) && ferror(stdin)) {
      swerr(proc, "error reading from stdin", 1); /* NOTREACHED */
   }
} /* GetStringFromUser() */


/*                                                                         *\
   -----------------------------------------------------------------------
			       Error Reporting
   -----------------------------------------------------------------------
\*                                                                         */

/**********
  * swerr *
  *********
   PURPOSE: Report errors detected during software execution to the standard
       error output device.
   PRE:  An error has been detected 
   POST: A detailed error message has been sent to the standard error
   output device.  The message includes the paramaters as descibed below.
   If the error_level was not 0 then the program has been halted with a 
   message and the error_level has been returned to the O/S as the program's
   exit value.

   NOTE: uses the module global character array `errorstring' 
*/
 void swerr(const char proc[],            /* name of procedure in which
                                             the error was detected    */
            const string err_mess,        /* additional error message  */
            const int error_level) {      /* if !0 exit() value        */

   extern string progname;
   extern int    errno;
   static char   swerr_string[MAX_ERRSTR]; 

   sprintf(swerr_string, "%s [%s()] %s", progname, proc, err_mess);
   if (0 == errno) {
      (void)fprintf(stderr, "%s\n", swerr_string);
   } else {
      perror(swerr_string);                                               
   }
   if (0 != error_level) {
      (void)fprintf(stderr, "%s halted due to unrecoverable error\n",progname);
      exit(error_level);
   }
 }

/*                                                                         *\
   -----------------------------------------------------------------------
                              Stream Routines
   -----------------------------------------------------------------------
\*                                                                         */

bool close_file(const string proc,
                const string fname,
                      FILE * handle)
{
   extern char errorstring[];

   if (fclose(handle) == EOF) {
      sprintf(errorstring, "cannot close file `%s'", fname);
      swerr(proc, errorstring, 0);
      return(FALSE);
   } else {
      open_files--;
#     if DEBUG > 1
         fprintf(stderr,"%s closed %s\n", proc, fname);
#     endif
      return(TRUE);
   }
} /* close_file() */



FILE * open_file(const string proc,
                 const string fname,
                 const string mode) 
{
           FILE * handle;
    extern char   errorstring[];

   if ((handle = fopen(fname, mode)) == NULL) {
#     ifdef DEBUG
         sprintf(errorstring, "cannot open file `%s'/open=%d", 
                 fname, open_files);
#     else
         sprintf(errorstring, "cannot open file `%s'", fname);
#     endif
      swerr(proc, errorstring, 0); 
#     ifdef SHOW_OPEN_FILES
      {
          int i = open_files;

          for(i=0; i < open_files; i++) {
             fprintf(stderr, "%d %s\n", i, open_names[i]);
          }
      }
      exit (-1);
#     endif
   } else {
#     ifdef SHOW_OPEN_FILES
         strncpy(open_names[open_files],fname,1023);
#        if DEBUG > 1
            fprintf(stderr,"%s opened %s\n", proc, fname);
#        endif
#     endif
      open_files++;
   }
   return(handle);
} /* open_file() */


bool file_exists(const string filename) {

   struct stat buf;

   return (-1 != stat(filename, &buf));
} /* file_exists() */


/*                                                                         *\
   -----------------------------------------------------------------------
                                User Input
   -----------------------------------------------------------------------
\*                                                                         */

bool yes(int *err) {
   char answer[3];

   if ((EOF != fputs("(y/n)? ", stdout)) && (NULL != fgets(answer, 3, stdin))){
      *err = 0;
      return ('y'==answer[0] || 'Y'==answer[0]);
   }
   *err = errno;
   clearerr(stdin);
   return(FALSE);
} /* yes() */


bool wait_for_enter_key(void) 
{
   #define LOTS_OF_JUNK (128)
   char   junk[LOTS_OF_JUNK + 1];

   if (EOF != (fputs("Press enter to continue...", stdout))) {
      fflush(stdout);
      return(NULL != fgets(junk, LOTS_OF_JUNK, stdin));
   } else {
      return (FALSE);
   }
   #undef LOTS_OF_JUNK
}/* wait_for_enter_key() */


/*
getchoice()
    PRE:  0 > low > high < 10^(MAX_DIGITS)-1
    POST: User has been prompted (possibly repeatedly) at stdout to enter an 
          integer between low and high.  When they have entered an invalid 
          value they have been told the allowable range.  If the default is 
          not 0 and they entered nothing, then the default was returned.
        Returns n ( low <= n <= high) on success OR
        Return  0 on error (error already reported)
*/
int getchoice(const elem_t low, 
              const elem_t high,
              const char   prompt[],
              const elem_t no_change)
{
   #define MAX_DIGITS 8
         char           proc[] = "getchoice";
         bool           done = FALSE;
         char           input[MAX_DIGITS] = "";
         short unsigned choice;

   do {
      fputs(prompt, stdout);
      fflush(stdout);
      if (NULL != fgets(input, sizeof(input), stdin)) {
         chop(input);
         if (input[0]) {
            choice = atoi(input);
            if ((choice >= low) && (choice <= high)) {
               done = TRUE;
            } else {
               printf("\nYou entered `%s'.\n", input);
               printf("Your choice must an integer between " \
                      ELEM_SPEC" and "ELEM_SPEC"\n",         \
                      low, high);
	    }
         } else {
            if (no_change) {
               choice = no_change;
               done = TRUE;
	    }
         }
      } else {
         swerr(proc, "error reading selection from user", 1);
         choice = 0;
         done = TRUE;
      }
   } while (!done);
   return(choice);
   #undef MAX_DIGITS
} /* getchoice() */


FILE * OutputFile(      char         outputname[],
                  const unsigned int maxlength) {

      const string proc = "OutputFile";
            int    yeserr;

      GetStringFromUser(outputname, maxlength, \
                   "Enter filename to use (or nothing to exit) ");
      chop(outputname);
      if (*outputname){ /* if it is a non-empty string */
         if (file_exists(outputname)) {
            puts("The file you have chosen already exists.");
            printf("Overwrite %s? ", outputname);
            if (! yes(&yeserr)) {
               return(NULL);
	    }
	 }
         return(open_file(proc,outputname,"w"));
      }
      return(NULL);
} /* OutputFile() */

