#include "types.h"
#include "proto.h"
#include "matrix.h"
#include "Hartley.h"
#include "Box-test.h"
#include "print.h"
#include "variance.h"

/* local macros */
#define Square(X) ((X) * (X))


/*                                                                         *\
   -----------------------------------------------------------------------
                             Compute Variances
   -----------------------------------------------------------------------
\*                                                                         */

void variance(const real       sum[],  /* The Xh. or X.j value */
                    Matrix *   mat,
              const matrix_dim numpartis,
              const matrix_dim numqueries,
              const bool       byparti,
                    real       var[])
{
   register matrix_dim parti, query;
   register real       total;
   register real       value;

   for (parti=0; parti < numpartis; parti++) {
      total=0.0;
      for (query=0; query < numqueries; query++) {
         value = (byparti)?mat_value(mat, query, parti)\
	                  :mat_value(mat, parti, query);
         total += Square(value);
      }
      total -= (Square(sum[parti]) / (real)numqueries);
      var[parti] = total / (real)(numqueries - (matrix_dim)1.0);
   }   
} /* variance() */


void ThreeWay_variance(const real       sum, /* sum of squares of all values */
                             Matrix *   mat[],  /* the raw data */
                       const matrix_dim numpartis,
                       const matrix_dim numqueries,
                       const matrix_dim numlevels,
                       real       var[MAX_PARTICIPANT][NUMofQUERIES])
{
   register matrix_dim parti, query, level;
   register real       total;
   register real       value;

   for (parti=0; parti < numpartis; parti++) {
      for (query=0; query < numqueries; query++) {
         if ((0 == parti) && (0 == query)) {
            total=0.0;
	 }
         for (level=0; level < numlevels; level++) {
            value = mat_value(mat[level], query, parti);
            total += Square(value);
	 }
         var[parti][query] = sum - (total / (real)numlevels);
         var[parti][query] /= (real)(numlevels - (matrix_dim)1.0);
      }
   }
} /* ThreeWay_variance() */


/*                                                                         *\
   -----------------------------------------------------------------------
                           Equality of Variances
   -----------------------------------------------------------------------
\*                                                                         */

bool variances_are_equal(      Matrix *   mat[],
                         const matrix_dim maxlevel,
                         const real       mean[],
                               real       variance[],
                         const bool       showall,
                               bool *     okay) 
{
   extern int        max_H_samples;
   const  char       proc[] ="variances_are_equal";
          matrix_dim num_samples = num_cols(mat[0]);
          matrix_dim dof         = num_rows(mat[0]) - 1;

   if (showall) {
      fputs(" ------------------------------------------------ ", stdout);
      NewLine(1, stdout);
      fputs("   Test for Equality of Participants' Variances   ", stdout);
      NewLine(1, stdout);
      fputs(" ------------------------------------------------ ", stdout);
      NewLine(1, stdout);
   }
   
   *okay = ((dof >= 2) && (maxlevel >= 1));
   if (*okay) {
      if (num_samples > max_H_samples) {
         if (showall) {
            printf("  more than %u samples -- using Box test", \
                   max_H_samples);
            NewLine(1, stdout);
         }
         return(G_test(mat[0], mean, variance, showall));
      } else {
         matrix_dim i, max, min;
         real       Fmax;

         if (showall) {
            printf("  not more than %u samples -- using Hartley test", \
                   max_H_samples);
            NewLine(1, stdout);
         }
         for (i=max=min=0; i < num_samples; i++) {
            if (variance[max] < variance[i]) {
               max = i;
	    } else 
            if (variance[min] > variance[i]) {
               min = i;
            }
	 }
         if (showall) {
            printf("   Fmax = "REAL_SPEC" / "REAL_SPEC, 
                   variance[max], variance[min]);
            NewLine(1, stdout);
	 }
         if (0.0 != variance[min]) {
            Fmax = variance[max] / variance[min];
            if (showall) {
               printf("   Fmax = "REAL_SPEC, Fmax);
               NewLine(1, stdout);
	    }
            return(Hartley_test(num_samples, dof, Fmax, showall));
	 } else {
            *okay = FALSE;
            return(TRUE);
	 }
      }
   } else {
      if (dof < 2) {
         swerr(proc, "Not enough degrees of freedom for a variance test", 0);
         if (showall) {
            fputs("Can't compare variances without at least two degrees " \
                  "of freedom.", stdout);
            NewLine(1, stdout);
	 }
      }
      return (FALSE);
   }
} /* variances_are_equal() */

